package se.ericsson.eto.norarc.javaframe;

import se.ericsson.eto.norarc.javaframe.debug.*;

import java.io.IOException;
import java.lang.reflect.*;
import java.util.*;


public class DebugScheduler extends Scheduler {
	
	  private Scheduler prevSched;
	  private Composite outer;
	  private static int indent;
	  private boolean connected;
	  private DebugWindow debugWindow;
	  private DebugControl dc;
	  private boolean paused;
	  
	  public boolean isPaused() {return paused;}
	  public void setConnection(boolean on) {
		  connected = on;
	  }
	  

	  /**
	   * Finds all statemachines in the system and moves them to this
	   * @param composite the outer composite of the system
	   */
	private void collectStateMachines (Composite composite) {
		  
		  try {
			  EditObject.setField(composite, "sched", this);
		  } catch (Exception e) {
			  throw new Error(e);
		  }

		  for (Object o : composite.activeObjectList) {

			  if (o instanceof Composite)
				  collectStateMachines((Composite) o);
			  else if (o instanceof StateMachine) {
				  StateMachine fsm = (StateMachine) o;
				  fsm.moveStateMachine(this);
				  
				  fsm.messageBox = new DebugMailBox(fsm.messageBox);
				  fsm.saveQueue = new DebugMailBox(fsm.saveQueue);

			  }
		  }
	  }
	
	  /**
	 * @param sched the scheduler currently running
	 * @param part an activeobject that can be used to find the outer composite of the system
	 */
	public DebugScheduler(Scheduler sched, ActiveObject part) {
		  this.prevSched = sched;
		  dc = DebugControl.getInstance();
		  paused = false;
		  try {
			  EditObject.init();
		  } catch (IOException e) {
			  throw new Error(e);
		  }
		  
		  // find the outer composite of the system
		  while (part.owner != null) part = part.owner;
		  if (part instanceof Composite) {
			  outer = (Composite) part;
			  collectStateMachines(outer);
			  debugWindow = new DebugWindow(this);
			  System.out.println("DebugScheduler started.");
			  trc = sched.trc;
		  } else {
			  throw new Error(
					  "Unable to find the outer composite of the system");
		  }
	  }
	  
	  /**
	   * The actual state machinery execution
	   */
	  public void run(){
	  // Eternal loop in order to make fsmList empty.
	  // If fsmList gets empty, the thread will sleep waiting for another thread
	  // to call readyFSM() with more work.
	  // Invariant: a FSM with nonempty messageBox is either curStateMachine xor
	  //            in fsmList
		  
		 
		  dc.setDebugScheduler(this);
	    while (true) {
	    	
	    	// TODO: window closed
	    	if (!debugWindow.isVisible()) {
	    		System.out.println("Window shutie");
	    		System.exit(0);
	    	}
	    	
	    	// check if DebugControl is trying to pause the scheduler
	    	while (true) {
	    		synchronized (this) {
	    			if (!dc.isPaused()) break;
	    			paused = true;
	    			notify();
	    			
	    			// wait until the DebugControl resumes execution
	    			try {
	    				wait();
	    			} catch (InterruptedException ie) {
	    				// the thread has been interrupted by Thread.interrupt()
	    				System.err.println("TRC Info("
		                    + Thread.currentThread().toString() + "interrupted" );
	    			}
	    		}
	    	}
	    	paused = false;
	    	
	      try {
	        synchronized (this) { // at this point: curStateMachine == null
	        	
	        	// TODO: wait if the connection is not ready?
	          if (fsmListFirst == null) {// || !connected) {
	        	  // no more FSM with nonempty messageBox
	            try {
	              wait(10000); // wait for new messages (or 10s)
	            } catch (InterruptedException ie) {
	              // the thread has been interrupted by Thread.interrupt()
	              System.err.println("TRC Info("
	                    + Thread.currentThread().toString() + "interrupted" );
	            }
	          } else { // get and remove the first FSM in fsmList
	            curStateMachine = fsmListFirst;
	            fsmListFirst = fsmListFirst.nextFSM;
	            if (fsmListFirst == null) fsmListLast = null;
	            curStateMachine.inSchedulerList = false;
	          }
	        }

	        if (curStateMachine != null) {
	        		if (curStateMachine.wantedScheduler == this) {// I am still the servant
	        			
	        			//process up to MAXMES messages from curStateMachine
	        			for (int i=0; i < MAXMES; i++) {
	        				if (dc.isPaused()) break;
	        				
	        				Message sig = curStateMachine.messageBox.removeFirst();
	        				if (sig == null)
	        					break;
	        				else {
	        					
	        					// log the transition to an event
	        					dc.startEvent(curStateMachine, sig);
	        					curStateMachine.exec(sig); // consume (or save) first message
	        					dc.stopEvent();
	        				}
	        			}
	        		
	        			
	            synchronized (this) {
	              // If still more messages, put curStateMachine into fsmList
	              if (curStateMachine.messageBox.getFirst() != null) {
	                if (fsmListLast == null)
	                  fsmListFirst = curStateMachine;
	                else
	                  fsmListLast.nextFSM = curStateMachine;
	                fsmListLast = curStateMachine;
	                curStateMachine.nextFSM = null;
	                curStateMachine.inSchedulerList = true;
	              }
	              curStateMachine = null;
	            }
	          } else { // curStateMachine wants a new scheduler
	            synchronized (curStateMachine) {
	              curStateMachine.myScheduler = curStateMachine.wantedScheduler;
	              if (curStateMachine.myScheduler != null)
	                curStateMachine.myScheduler.readyFSM(curStateMachine);
	            }
	            curStateMachine = null;
	          }
	        }
	      } catch (Exception e) { //Do not stop execution due to any exceptions
	        e.printStackTrace();
	        System.out.println("Continuing execution");
	      }
	    }
	  }
	  
	  /**
	   * Enables the given FSM for inspection by this Scheduler.
	   * @param sm assumed to be != null.
	   */
	  synchronized
	  protected void readyFSM(StateMachine sm) {
		  // TODO: StateMachine terminated or created
		  if (sm.wantedScheduler == null)
			  // sm has terminated
			  dc.eventTerminate(sm);
		  else if (sm.wantedScheduler == this) {
				Message m = sm.messageBox.getFirst();
				if (m != null && m instanceof StartMessage
						&& m == sm.messageBox.lastMessage)
					// sm has been created
					dc.eventCreate(sm);
		  }
			  
		  super.readyFSM(sm);
	  }
	  
}
