package se.ericsson.eto.norarc.javaframe.debug;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import se.ericsson.eto.norarc.javaframe.DebugControl;

public class Connection implements Runnable {
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private boolean canceled = false;
	private ServerSocket server;
	private Socket client;
	private DebugWindow debugWindow;
	private DebugControl dc;
	int port;
	boolean valid;
	
	public void cancel() {
		synchronized(this) {
			canceled = true;
			closeAll();
			dc.disConnected();
		}
		
	}
	
	public Connection(DebugWindow debugWindow, int port) {
		valid = false;
		this.port = port;
		this.debugWindow = debugWindow;
		dc = DebugControl.getInstance();
	}
	
	public void run() {
		try {
			server = new ServerSocket(port);
		} catch (IOException e) {
			debugWindow.setText("Fail");
			e.printStackTrace();
			debugWindow.setEnabled(true);
		}
		debugWindow.accepting(port);
		try {
			client = server.accept();
		} catch (IOException e) {
			synchronized (this) {
				if (!canceled) {
					e.printStackTrace();
					closeAll();
					debugWindow.failed();
					return;
				}
			}
		}
		
		// connection is set up
		boolean success = false;
		synchronized (this) {
			
			if (canceled) return;
			try {
				in = new ObjectInputStream(client.getInputStream());
				out = new ObjectOutputStream(client.getOutputStream());
				success = true;
			} catch (IOException e) {
				e.printStackTrace();
			}
			if (!success) {
				closeAll();
				debugWindow.failed();
			} else {
				debugWindow.setText("Win");
				valid = true;
				dc.connected(this);
			}
		}
	}
	
	private void closeAll() {
		
		if (in != null) {
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			in = null;
		}
		if (out != null) {
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			out = null;
		}
		if (client != null) {
			try {
				client.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			client = null;
		}
		if (server != null) {
			try {
				server.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			server = null;
		}
	}
	
	public Object readObject() {
		out("readObject");
		synchronized (in) {
			if (!valid)
				return null;
			Object o = null;
			try {
				o = in.readObject();
			} catch (Exception e) {
				e.printStackTrace();
				valid = false;
				closeAll();
				dc.disConnected();
				debugWindow.failed();
			}
			out(o + " recieved");
			return o;
		}
	}

	public void writeObject(Object o) {
		out("writeObject:");
		synchronized (out) {
			out("Sending " + o);
			if (!valid)
				return;
			try {
				out.writeObject(o);
				out(o + " sent");
			} catch (Exception e) {
				e.printStackTrace();
				closeAll();
				dc.disConnected();
				debugWindow.failed();
				valid = false;
			}
		}
	}

	private void out(Object o) {
		System.out.println("Connection: " + o);
	}
}
