package se.ericsson.eto.norarc.javaframe.debug;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;

import se.ericsson.eto.norarc.javaframe.DebugControl;
import se.ericsson.eto.norarc.javaframe.DebugScheduler;
import se.ericsson.eto.norarc.javaframe.Event;

public class DebugWindow extends JFrame {

	private DebugControl dc;
	private DebugScheduler sched;
	private DebugWindow thisDebugWindow;
	// Connection Window
	private JButton connectButton;
	private JButton cancelConnectButton;
	private JTextField portField;
	private JLabel connectionStatus;

	// socket stuff

	// pause/resume etc
	private JButton pauseButton;
	private JButton resumeButton;
	private JButton nextButton;
	private JButton prevButton;

	private class ConnectButtonListener implements ActionListener {

		Connection con;

		private void connect() {
			connectButton.setEnabled(false);
			int port = -1;
			boolean validPort = false;
			try {

				port = Integer.parseInt(portField.getText());
				if (port >= 0 && port <= 65535)
					validPort = true;
			} catch (NumberFormatException e) {
			}

			if (!validPort) {
				connectionStatus.setText(portField.getText()
						+ " not a valid port number");
				connectButton.setEnabled(true);
				return;
			}
			con = new Connection(thisDebugWindow, port);
			Thread t = new Thread(con);
			t.start();

		}

		private void cancel() {

			con.cancel();

			connectionStatus.setText("Connection canceled");
			cancelConnectButton.setEnabled(false);
			connectButton.setEnabled(true);

		}

		public void actionPerformed(ActionEvent ae) {
			if (ae.getSource() == connectButton)
				connect();
			else if (ae.getSource() == cancelConnectButton)
				cancel();

		}

	}

	private class NavButtonListener implements ActionListener {
		private void setNP() {
			nextButton.setEnabled(!dc.emptyFuture());
			prevButton.setEnabled(!dc.emptyPast());
		}

		private void pause() {
			pauseButton.setEnabled(false);
			dc.pause();
			resumeButton.setEnabled(true);
			setNP();
		}

		private void resume() {
			resumeButton.setEnabled(false);
			dc.resume();
			pauseButton.setEnabled(true);
			nextButton.setEnabled(false);
			prevButton.setEnabled(false);
		}

		private void next() {
			dc.next();
			setNP();
		}

		private void previous() {
			dc.previous();
			setNP();
		}

		public void actionPerformed(ActionEvent ae) {
			Object s = ae.getSource();
			if (s == pauseButton)
				pause();
			else if (s == resumeButton)
				resume();
			else if (s == nextButton)
				next();
			else if (s == prevButton)
				previous();

		}

	}

	private Component connectionContainer() {
		Container c = new Container();
		c.setLayout(new FlowLayout());

		c.add(new JLabel("Port:"));
		c.add(portField);
		c.add(connectButton);
		c.add(cancelConnectButton);
		c.add(connectionStatus);
		connectionStatus.setText("Everything is fine and everyone is happy");
		return c;
	}

	private Component navigationContainer() {
		Container c = new Container();
		c.setLayout(new FlowLayout());
		c.add(pauseButton);
		c.add(resumeButton);
		c.add(prevButton);
		c.add(nextButton);
		return c;
	}

	public void setText(String s) {
		connectionStatus.setText(s);
	}

	public void accepting(int port) {
		connectionStatus
				.setText("Accepting connection on port " + port + "...");
		cancelConnectButton.setEnabled(true);
	}

	public void failed() {
		connectionStatus.setText("Fail");
		connectButton.setEnabled(true);
		cancelConnectButton.setEnabled(false);
	}

	public DebugWindow(DebugScheduler sched) throws HeadlessException {
		super("JFDebug");
		thisDebugWindow = this;
		dc = DebugControl.getInstance();
		this.sched = sched;

		// connectionWindow
		connectButton = new JButton("Open");
		cancelConnectButton = new JButton("Cancel");
		portField = new JTextField("12345", 10);
		ConnectButtonListener cbl = new ConnectButtonListener();
		cancelConnectButton.addActionListener(cbl);
		cancelConnectButton.setEnabled(false);
		connectButton.addActionListener(cbl);
		connectionStatus = new JLabel();
		Container content = new Container();
		content.setLayout(new BorderLayout());
		content.add(connectionContainer(), BorderLayout.NORTH);

		pauseButton = new JButton("Pause");
		resumeButton = new JButton("Resume");
		resumeButton.setEnabled(false);
		nextButton = new JButton("Next");
		nextButton.setEnabled(false);
		prevButton = new JButton("Previous");
		prevButton.setEnabled(false);
		NavButtonListener nbl = new NavButtonListener();
		pauseButton.addActionListener(nbl);
		resumeButton.addActionListener(nbl);
		nextButton.addActionListener(nbl);
		prevButton.addActionListener(nbl);

		content.add(navigationContainer(), BorderLayout.SOUTH);
		Container cPane = getContentPane();
		cPane.setLayout(new BorderLayout());
		cPane.add(content, BorderLayout.NORTH);
		pack();
		setVisible(true);
		dc.setDebugWindow(this);
	}

	
	private JScrollPane eventScrollPane;

	public void setEvent(Event e) {
		Container cPane = getContentPane();
		if (eventScrollPane != null) cPane.remove(eventScrollPane);
		eventScrollPane = null;
		if (e == null) {
			pack();
			return;
		}
		StateMachineClassData cData = e.getFsmData().getClassData();
		String cNames[] = {"attribute", "before", "after"};
		Object data[][] = new Object[cData.getAttributes().length + 1][3];
		data[0][0] = "state";
		if (e.getBeforeState() != null)
			data[0][1] = e.getBeforeState().stateName;
		else data[0][1] = "initial";
		if (e.getAfterState() != null)
			data[0][2] = e.getAfterState().stateName;
		else data[0][2] = "?";
		System.out.println(" --- "+(cData.getAttributes().length + 1));
		System.out.println(" --- "+data.length);
		for (int i = 0; i < cData.getAttributes().length; i++) {
			data[i+1][0] = cData.getAttributes()[i];
			data[i+1][1] = e.getBeforeAttr()[i];
			data[i+1][2] = e.getAfterAttr()[i];
		}
		
		JTable table = new JTable(data, cNames);
		eventScrollPane = new JScrollPane(table);
		cPane.add(eventScrollPane);
		pack();

	}

}
